#! /usr/bin/env python3
__author__ = 'Lene Preuss <lene.preuss@gmail.com>'

from typing import List

import click

from mafe.scan import Scanner, TrackData, DEFAULT_STORE_EVERY


@click.command()
@click.option(
    '--base-folders', '-f', multiple=True, required=True, help='Directory to scan'
)
@click.option(
    '--tracks-csv', '-t', default='tracks.csv.bz2', help='CSV file containing tracks'
)
@click.option('--max-track-length', '-m', type=int, help='Maximum track length, in seconds')
@click.option(
    '--quiet', '-q', is_flag=True, default=False, help='Suppress warnings and progress messages'
)
@click.option(
    '--store-every', '-s', type=int, default=DEFAULT_STORE_EVERY, help='Store every n tracks'
)
def scan_folders(
        base_folders: List[str], tracks_csv: str, max_track_length: int,
        quiet: bool, store_every: int
) -> None:
    if max_track_length is not None:
        TrackData.set_max_track_duration(max_track_length)

    scanner = Scanner(tracks_csv, base_folders, quiet, store_every)
    scanner.run()



#! /usr/bin/env python3
__author__ = 'Lene Preuss <lene.preuss@gmail.com>'

import click
import pandas as pd

from mafe.process import DistanceCalculator, normalize as proc_normalize, norm_mean, Cluster
from mafe.process.pca import Reducer


@click.group()
@click.option(
    '--tracks-csv', '-t', required=True, help='CSV file containing tracks'
)
@click.option(
    '--output', '-o', required=True, help='CSV file containing distances between the tracks'
)
@click.option(
    '--verbose', '-v', is_flag=True, default=False, help='Suppress warnings and progress messages'
)
@click.pass_context
def cli(ctx, tracks_csv: str, output: str, verbose: bool):
    ctx.ensure_object(dict)
    ctx.obj['tracks_csv'] = tracks_csv
    ctx.obj['output'] = output
    ctx.obj['verbose'] = verbose


@cli.command()
@click.pass_context
def normalize(ctx) -> None:
    data = pd.read_csv(ctx.obj['tracks_csv'])
    normalized = proc_normalize(data, func=norm_mean)
    normalized.to_csv(ctx.obj['output'])


@cli.command()
@click.pass_context
def pca(ctx) -> None:
    reducer = Reducer(ctx.obj['tracks_csv'])
    reducer.run().to_csv(ctx.obj['output'])


@cli.command()
@click.pass_context
def distance(ctx) -> None:
    calculator = DistanceCalculator(ctx.obj['tracks_csv'], ctx.obj['output'])
    calculator.run(verbose=(ctx.obj['verbose']))


@cli.command()
@click.option(
    '--num-clusters', '-n', required=True, type=int, help='Number of clusters to compute'
)
@click.pass_context
def cluster(ctx, num_clusters: int) -> None:
    clusterer = Cluster(ctx.obj['tracks_csv'])
    clusterer.run(num_clusters).to_csv(ctx.obj['output'])



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['mafe', 'mafe.mafe', 'mafe.process', 'mafe.scan']

package_data = \
{'': ['*']}

install_requires = \
['audio-metadata>=0.11.1,<0.12.0',
 'audioread>=2.1.8,<3.0.0',
 'click>=7.1.2,<8.0.0',
 'librosa>=0.7.2,<0.8.0',
 'numba>=0.49.1,<0.50.0',
 'numpy>=1.18.5,<2.0.0',
 'pandas>=1.0.4,<2.0.0',
 'python-magic>=0.4.18,<0.5.0',
 'scikit-learn>=0.23.1,<0.24.0',
 'scipy>=1.4.1,<2.0.0']

entry_points = \
{'console_scripts': ['process = mafe.mafe.mafe_process:cli',
                     'scan = mafe.mafe.mafe_scan:scan_folders']}

setup_kwargs = {
    'name': 'mafe',
    'version': '0.1.15',
    'description': 'Music Audio Feature Extractor',
    'long_description': '# Music Audio Feature Extractor\n\n## `mafe_scan.py`\n\n```bash\n$ mafe_scan.py --help\nUsage: mafe_scan.py [OPTIONS]\n\nOptions:\n  -f, --base-folders TEXT  Directory to scan  [required]\n  -p, --parallel           Use parallel execution\n  -q, --quiet              Suppress warnings and progress messages\n  --help                   Show this message and exit.\n```',
    'author': 'Lene Preuss',
    'author_email': 'lene.preuss@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://gitlab.com/lilacashes/mafe',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



__author__ = 'Lene Preuss <lene.preuss@gmail.com>'

from typing import Tuple

import pandas as pd
from sklearn.cluster import KMeans

from mafe.scan import TrackData


def k_means(
        tracks: pd.DataFrame, n_clusters: int = 8, max_iter: int = 300
) -> KMeans:
    kmeans = KMeans(n_clusters=n_clusters, max_iter=max_iter).fit(tracks)
    return kmeans


class Cluster:  # pylint: disable=too-few-public-methods

    def __init__(
            self, tracks_csv: str, text_cols: Tuple[str, ...] = None
    ) -> None:
        self.tracks = pd.read_csv(tracks_csv)
        self.text_cols = list(TrackData.STRING_COLUMNS) if text_cols is None else text_cols

    def run(self, n_clusters: int) -> pd.DataFrame:
        numbers_only = self.tracks.drop(self.text_cols, axis='columns').dropna(axis='columns')
        kmeans = k_means(numbers_only, n_clusters)
        self.tracks.insert(1, 'cluster', kmeans.labels_, True)
        return self.tracks
